@@ -1,30 +1,31 @@
 /************************************************************************
- * $Id: astro.c,v 1.8 2004/12/10 14:08:04 thamer Exp $
+ * $Id: astro.c 2185 2009-03-09 13:56:29Z thamer $
  *
  * ------------
  * Description:
  * ------------
- *  Copyright (c) 2003, Arabeyes, Thamer Mahmoud
+ *  Copyright (c) 2003-2006, 2009 Arabeyes, Thamer Mahmoud
  *
  *  A full featured Muslim Prayer Times calculator
  *
- *  Most of the astronomical values and formulas used in this file are
- *  based on the Jean Meeus subset of the VSOP87 planetary theory.
+ *  Most of the astronomical values and formulas used in this file are based
+ *  upon a subset of the VSOP87 planetary theory developed by Jean Meeus. Page
+ *  and formula numbers in-line below are references to his book: Astronomical
+ *  Algorithms. Willmann-Bell, second edition, 1998.
  *
  * -----------------
  * Revision Details:    (Updated by Revision Control System)
  * -----------------
- *  $Date: 2004/12/10 14:08:04 $
+ *  $Date: 2009-03-09 16:56:29 +0300 (Mon, 09 Mar 2009) $
  *  $Author: thamer $
- *  $Revision: 1.8 $
- *  $Source: /home/arabeyes/cvs/projects/itl/libs/prayertime/src/astro.c,v $
+ *  $Revision: 2185 $
+ *  $Source$
  *
  * (www.arabeyes.org - under LGPL license - see COPYING file)
  ************************************************************************/
 
 #include "astro.h"
 
-
 typedef struct
 {
     double dec;
@@ -35,20 +36,229 @@ typedef struct
         
 } AstroDay ;
 
-
+enum Type  { SUNRISE,
+             SUNSET };
+ 
 static void computeAstroDay(double JD, AstroDay* astroday);
-static void computeTopAstro(const Location* loc, const Astro* astro, Astro* topAstro);
+static void computeTopAstro(const Location* loc, const Astro* astro, Astro* tastro);
+static double getRiseSet (const Location* loc, const Astro* astro, int type);
+static double getRefraction(const Location* loc, double sunAlt);
+static double limitAngle180(double L);
+static double limitAngle(double L);
+static double limitAngle1(double L);
+static double limitAngle180between(double L);
+static double getYearFromJulian(double jd);
+
+double getSunrise (const Location* loc, const Astro* tastro)
+{
+    return getRiseSet (loc, tastro, SUNRISE);
+} 
+
+double getSunset (const Location* loc, const Astro* tastro)
+{
+    return getRiseSet (loc, tastro, SUNSET);
+} 
+
+double getTransit(double lon, const Astro* tastro)
+{
+
+    double M, sidG;
+    double ra0=tastro->ra[0], ra2=tastro->ra[2];
+    double A, H;
+
+    M = ((tastro->ra[1] - lon - tastro->sid[1]) / 360.0);
+    M = limitAngle1(M);
+
+    /* Sidereal time at Greenwich (p. 103) */ 
+    sidG =  tastro->sid[1] + 360.985647 * M;
+
+    if (tastro->ra[1] > 350 && tastro->ra[2] < 10)
+        ra2 += 360;
+    if (tastro->ra[0] > 350 && tastro->ra[1] < 10)
+        ra0 = 0;
+    
+    /* Interpolation of 1-day intervals (pp. 24-25) */
+    A = tastro->ra[1] + 
+	(M * ((tastro->ra[1] - ra0) + ( ra2 - tastro->ra[1]) + 
+	      (( ra2 - tastro->ra[1]) - (tastro->ra[1] - ra0)) * M) / 2.0 );
+
+    H =  limitAngle180between(sidG + lon - A);
+
+    return  24.0 * (M - (H/360.0));
+}
+
+static double getRiseSet (const Location* loc, const Astro* tastro, int type)
+{
+    /* p. 101 */
+    double lhour, M, sidG, ra0, ra2;
+    double A, B, H, sunAlt, delM, tH, rDec, rLat, rB;
+    double part1, part2, part3;
+
+    rDec = DEG_TO_RAD(tastro->dec[1]);
+    rLat = DEG_TO_RAD(loc->degreeLat);
+
+    ra0=tastro->ra[0];
+    ra2=tastro->ra[2];
+    
+    /* Compute the hour angle */
+    part1 = sin(DEG_TO_RAD(CENTER_OF_SUN_ANGLE)) - (sin (rLat) * sin (rDec));
+    part2 = cos (rLat) * cos (rDec);
+    part3 = part1 / part2;
+
+    if  ( part3 < -INVALID_TRIGGER || part3 > INVALID_TRIGGER)
+        return 99;
+
+    lhour = limitAngle180 (( RAD_TO_DEG (acos (part3))));
+ 
+    /* Eastern Longitudes are positive throughout this file. */ 
+    M = ((tastro->ra[1] - loc->degreeLong - tastro->sid[1]) / 360.0);
+
+    if (type == SUNRISE)
+        M = M - (lhour/360.0);
+    if (type == SUNSET)
+        M = M + (lhour/360.0);
+        
+    M = limitAngle1(M);
+
+    /* Sidereal time at Greenwich (p. 103) */ 
+    sidG = limitAngle(tastro->sid[1] + 360.985647 * M);
+
+    ra0 = tastro->ra[0];
+    ra2 = tastro->ra[2];
+
+    if (tastro->ra[1] > 350 && tastro->ra[2] < 10)
+        ra2 += 360;
+    if (tastro->ra[0] > 350 && tastro->ra[1] < 10)
+        ra0 = 0;
+
+    /* Interpolation of 1-day intervals (pp. 24-25) */
+    A = tastro->ra[1] + (M * (( tastro->ra[1] - ra0) +
+                             (ra2 - tastro->ra[1] ) +
+                             (( ra2 - tastro->ra[1] ) -
+                              ( tastro->ra[1]  -  ra0)) * M) / 2.0 );
+
+    B = tastro->dec[1] + (M * ((tastro->dec[1] - tastro->dec[0]) + 
+                              (tastro->dec[2] - tastro->dec[1]) + 
+                              ((tastro->dec[2] - tastro->dec[1]) -  
+                               (tastro->dec[1] - tastro->dec[0])) * M) / 2.0 );
+    rB = DEG_TO_RAD(B);
+
+    H = limitAngle180between(sidG + loc->degreeLong - A);
+
+    tH =  DEG_TO_RAD(H) - tastro->dra[1];
+
+    /* Airless Sun's altitude at local horizontal coordinates (p. 93, 13.6) */
+    sunAlt = RAD_TO_DEG(asin (  sin(rLat) * sin(rB) 
+                                + cos(rLat) * cos(rB) 
+                                * cos(tH) ));
+
+    sunAlt += getRefraction(loc, sunAlt);
+
+    /* (p. 103) */
+    delM = (sunAlt - CENTER_OF_SUN_ANGLE) / (360.0 * cos(rB) * cos(rLat)
+					     * sin(tH));
+
+    return  (M + delM) * 24.0;
+
+}
+
 
 double getRefraction(const Location* loc, double sunAlt)
 {
     double part1, part2;
 
     part1 = (loc->pressure/1010.0) * (283/(273 + loc->temperature));
-    part2 = 1 / tan(DEG_TO_RAD(sunAlt + (7.31/(sunAlt + 4.4)))) + 0.0013515;
+    part2 = 1.02 / (RAD_TO_DEG(tan(DEG_TO_RAD(sunAlt + (10.3/(sunAlt + 5.11))))) + 0.0019279);
 
     return (part1 * part2) / 60.0;
 }
 
+/* const double DT[][5]={
+    {121, 112, 103, 95, 88},
+    {82, 77, 72, 68, 63},
+    {60, 56, 53, 51, 48},
+    {46, 44, 42, 40, 38},
+    {35, 33, 31, 29, 26},
+    {24, 22, 20, 18, 16},
+    {14, 12, 11, 10, 9},
+    {8, 7, 7, 7, 7},
+    {7, 7, 8, 8, 9},
+    {9, 9, 9, 9, 10},
+    {10, 10, 10, 10, 10},
+    {10, 10, 11, 11, 11},
+    {11, 11, 12, 12, 12},
+    {12, 13, 13, 13, 14},
+    {14, 14, 14, 15, 15},
+    {15, 15, 15, 16, 16},
+    {16, 16, 16, 16, 16},
+    {16, 15, 15, 14, 13},
+    {13.1, 12.5, 12.2, 12, 12},
+    {12, 12, 12, 12, 11.9},
+    {11.6, 11, 10.2, 9.2, 8.2},
+    {7.1, 6.2, 5.6, 5.4, 5.3},
+    {5.4, 5.6, 5.9, 6.2, 6.5},
+    {6.8, 7.1, 7.3, 7.5, 7.6},
+    {7.7, 7.3, 6.2, 5.2, 2.7},
+    {1.4, -1.2, -2.8, -3.8, -4.8},
+    {-5.5, -5.3, -5.6, -5.7, -5.9},
+    {-6.0, -6.3, -6.5, -6.2, -4.7},
+    {-2.8, -0.1, 2.6, 5.3, 7.7},
+    {10.4, 13.3, 16, 18.2, 20.2},
+    {21.1, 22.4, 23.5, 23.8, 24.3},
+    {24, 23.9, 23.9, 23.7, 24},
+    {24.3, 25.3, 26.2, 27.3, 28.2},
+    {29.1, 30, 30.7, 31.4, 32.2},
+    {33.1, 34, 35, 36.5, 38.3},
+    {40.2, 42.2, 44.5, 46.5, 48.5},
+    {50.5, 52.2, 53.8, 54.9, 55.8},
+    {56.9, 58.3, 60, 61.6, 63}, /* 1990-1998 */
+/* };  */
+
+const double DT2[]={
+    63.4673, 63.8285, 64.0908, 64.2998, 64.4734, /* 1999-2003 */
+    64.5736, 64.7052, 64.8452, 65.1464, 65.4574, /* 2004-2008 */
+    65.7768,                                     /* 2009 */
+    66.5, 67.1, 68, 68, 69,                      /* 2010-2014 predictions */
+    69, 70, 70                                   /* 2015-2017 predictions */
+};
+ 
+static double computeDeltaT(double year)
+{
+    int i;
+    double tempdt;
+    /* pp. 78-80 */
+    double t = (year - 2000) / 100.0;
+    if (year < 948)
+	return 2177 + (497 * t) + (44.1 * pow (t, 2));
+    else if (year >= 1620 && year <= 1998)
+	return 0; /* FIXIT: Support historical delta-t values for years before
+		   * 1998. In the DT table above, each line represents 5 even
+		   * years in the range 1620-1998. We should first complete the
+		   * table to include data for both odd and even years. */
+    else if ((year > 1998 && year < 2100) || year < 1620)
+    {
+	/* FIXIT: The "2017" found below this comment should be changed to
+	   reflect the last year added to the DT2 table. */
+	if (year >= 1999 && year <= 2017) {
+	    i = year-1999;
+	    return DT2[i];
+	}
+	/* FIXIT: As of 2007, the two formulas given by Meeus seems to be off by
+	   many seconds when compared to observed values found at
+	   <http://maia.usno.navy.mil>. The DT2 table overrides these values
+	   with observed and predicted ones for delta-t (on January, other
+	   months not yet supported). Extrapolated (and wrong) values are still
+	   used for years after 2017. */
+	else tempdt = 102 + (102 * t) + (25.3 * pow (t, 2));
+
+	if (year >= 2000)
+	    return tempdt + (0.37 * (year - 2100));
+	else return tempdt;
+    }
+    return 0;
+}
+
+
 double getJulianDay(const Date* date, double gmt)
 {
     double jdB=0, jdY, jdM, JD;
@@ -69,9 +279,10 @@ double getJulianDay(const Date* date, double gmt)
                                  ((date->month == 10) && (date->day >= 4)))))
         jdB = 2 - floor(jdY/100.0) + floor((jdY/100.0)/4.0);
 
-    JD = floor(365.25 * (jdY + 4716.0)) + floor(30.6001 * ( jdM + 1))
-        + (date->day + (-gmt)/24.0)  + jdB - 1524.5 ;
+    JD = floor(365.25 * (jdY + 4716.0)) + floor(30.6001 * (jdM + 1))
+        + (date->day + (-gmt)/24.0) + jdB - 1524.5 ;
 
+    JD = JD + (computeDeltaT(date->year) / 86400.0);
     return JD;
 
 }
@@ -102,7 +313,7 @@ const double L0[64][3]={
     {243, 0.345, 5486.778},
     {206, 4.806, 2544.314},
     {205, 1.869, 5573.143},
-    {202, 2.4458, 6069.777},
+    {202, 2.458, 6069.777},
     {156, 0.833, 213.299},
     {132, 3.411, 2942.463},
     {126, 1.083, 20.775},
@@ -117,7 +328,7 @@ const double L0[64][3]={
     {85, 3.67, 71430.7},
     {80, 1.81, 17260.15},
     {79, 3.04, 12036.46},
-    {71, 1.76, 5088.63},
+    {75, 1.76, 5088.63},
     {74, 3.5, 3154.69},
     {74, 4.68, 801.82},
     {70, 0.83, 9437.76},
@@ -171,7 +382,7 @@ const double L1[][3]={
     {12, 5.27, 1194.45},
     {12, 2.08, 4694},
     {11, 0.77, 553.57},
-    {10, 1.3, 3286.6},
+    {10, 1.3, 6286.6},
     {10, 4.24, 1349.87},
     {9, 2.7, 242.73},
     {9, 5.64, 951.72},
@@ -214,9 +425,9 @@ const double L3[][3]={
 };
 
 const double L4[][3]={
-    {114.0, 3.142, 0.0},
-    {8.0, 4.13, 6283.08},
-    {1.0, 3.84, 12566.15}
+    {114, 3.142, 0},
+    {8, 4.13, 6283.08},
+    {1, 3.84, 12566.15}
 };
 
 const double L5[][3]={
@@ -259,7 +470,7 @@ const double R0[][3]={
     {98, 0.89, 6069.78},
     {86, 5.69, 15720.84},
     {86, 1.27, 161000.69},
-    {85, 0.27, 17260.15},
+    {65, 0.27, 17260.15},
     {63, 0.92, 529.69},
     {57, 2.01, 83996.85},
     {56, 5.24, 71430.7},
@@ -302,7 +513,7 @@ const double R2[][3]={
     {12, 3.14, 0},
     {9, 3.63, 77713.77},
     {6, 1.87, 5573.14},
-    {3, 5.47, 18849}
+    {3, 5.47, 18849.23}
 
 };
 
@@ -315,7 +526,7 @@ const double R4[][3]={
     {4, 2.56, 6283.08}
 };
 
-const double PE[][4]={
+const double PN[][4]={
     {-171996, -174.2, 92025, 8.9},
     {-13187, -1.6, 5736, -3.1},
     {-2274, -0.2, 977, -0.5},
@@ -381,7 +592,7 @@ const double PE[][4]={
     {-3, 0, 0, 0}
 };
 
-const int SINCOEFF[][5]={
+const int COEFF[][5]={
     {0, 0, 0, 0, 1},
     {-2, 0, 0, 2, 2},
     {0, 0, 0, 2, 2},
@@ -454,6 +665,7 @@ void getAstroValuesByDay(double julianDay, const Location* loc, Astro* astro,
 
     if (astro->jd == julianDay-1)
     {
+        /* Copy cached values */
         astro->ra[0] = astro->ra[1];
         astro->ra[1] = astro->ra[2];
         astro->dec[0] = astro->dec[1];
@@ -464,6 +676,7 @@ void getAstroValuesByDay(double julianDay, const Location* loc, Astro* astro,
         astro->dra[1] = astro->dra[2];
         astro->rsum[0] = astro->rsum[1];
         astro->rsum[1] = astro->rsum[2];
+        /* Compute next day values */
         computeAstroDay(julianDay+1, &ad);
         astro->ra[2] = ad.ra;
         astro->dec[2] = ad.dec;
@@ -473,7 +686,7 @@ void getAstroValuesByDay(double julianDay, const Location* loc, Astro* astro,
     }
     else if (astro->jd == julianDay + 1)
     {
-
+        /* Copy cached values */
         astro->ra[2] = astro->ra[1];
         astro->ra[1] = astro->ra[0];
         astro->dec[2] = astro->dec[1];
@@ -484,6 +697,7 @@ void getAstroValuesByDay(double julianDay, const Location* loc, Astro* astro,
         astro->dra[1] = astro->dra[0];
         astro->rsum[2] = astro->rsum[1];
         astro->rsum[1] = astro->rsum[0];
+        /* Compute previous day values */
         computeAstroDay(julianDay-1, &ad);
         astro->ra[0] = ad.ra;
         astro->dec[0] = ad.dec;
@@ -494,7 +708,7 @@ void getAstroValuesByDay(double julianDay, const Location* loc, Astro* astro,
 
     } else if (astro->jd != julianDay)
     {
-        
+        /* Compute 3 day values */
         computeAstroDay(julianDay-1, &ad);
         astro->ra[0] = ad.ra;
         astro->dec[0] = ad.dec;
@@ -526,14 +740,14 @@ void computeAstroDay(double JD, AstroDay* astroday)
 {
 
     int i =0;
-    double R, Gg, G;
+    double R, Gg, rGg, G;
 
     double tL, L;
     double tB, B;
 
-    double X0, X1, X2, X3, X4;
+    double D, M, M1, F, O;
 
-    double U, E0, E, lamda, V0, V;
+    double U, E0, E, rE, lambda, rLambda, V0, V;
 
     double RAn, RAd, RA, DEC;
 
@@ -541,17 +755,22 @@ void computeAstroDay(double JD, AstroDay* astroday)
     double R0sum=0, R1sum=0, R2sum=0, R3sum=0, R4sum=0;
     double L0sum=0, L1sum=0, L2sum=0, L3sum=0, L4sum=0, L5sum=0;
 
-    double xsum=0, psi=0, epsilon=0;
+    double PNsum=0, psi=0, epsilon=0;
     double deltaPsi, deltaEps;
 
-    double JC = (JD - 2451545)/36525.0;                                                 
-    double JM = JC/10.0 ; 
+    double JC = (JD - 2451545)/36525.0;                                             
+    double JM = JC/10.0; 
     double JM2 = pow (JM, 2);
     double JM3 = pow (JM, 3);
     double JM4 = pow (JM, 4);
     double JM5 = pow (JM, 5);
 
+    /* FIXIT: By default, the getJulianDay function returns JDE rather then JD,
+     * make sure this is accurate, and works right in last-day-of-year
+     * circumstances.  */
+    double JDE = JD;
 
+    double T = (JDE - 2451545)/36525.0;
 
     for(i=0; i < 64; i++)
         L0sum += L0[i][0] * cos(L0[i][1] + L0[i][2] * JM);
@@ -594,55 +813,62 @@ void computeAstroDay(double JD, AstroDay* astroday)
 
     R = (R0sum + (R1sum * JM) + (R2sum * JM2)
          + (R3sum * JM3) + (R4sum * JM4)) / pow (10, 8);
-
+    
     G = limitAngle((L + 180));
     Gg = -B;
-
-    X0 = 297.85036 + (445267.111480 * JC) -  (0.0019142 * pow (JC, 2)) +
-        pow (JC, 3)/189474.0;
-    X1 = 357.52772 + (35999.050340 * JC) -  (0.0001603 * pow (JC, 2)) -  
-        pow (JC, 3)/300000.0;
-    X2 = 134.96298 + (477198.867398 * JC) +  (0.0086972 * pow (JC, 2)) +  
-        pow (JC, 3)/56250.0;
-    X3 = 93.27191 + (483202.017538 * JC) -  ( 0.0036825 * pow (JC, 2)) +  
-        pow (JC, 3)/327270.0;
-    X4 = 125.04452 - (1934.136261 * JC) + (0.0020708 * pow (JC, 2)) +  
-        pow (JC, 3)/450000.0;
-
-
+    rGg = DEG_TO_RAD(Gg);
+    /* Compute the fundamental arguments (p. 144) */
+    D = 297.85036 + (445267.111480 * T) -  (0.0019142 * pow (T, 2)) +
+        (pow (T, 3)/189474.0);
+    M = 357.52772 + (35999.050340 * T) -  (0.0001603 * pow (T, 2)) -  
+        (pow (T, 3)/300000.0);
+    M1 = 134.96298 + (477198.867398 * T) +  (0.0086972 * pow (T, 2)) +  
+        (pow (T, 3)/56250.0);
+    F = 93.27191 + (483202.017538 * T) -  ( 0.0036825 * pow (T, 2)) +  
+        (pow (T, 3)/327270.0);
+    O = 125.04452 - (1934.136261 * T) + (0.0020708 * pow (T, 2)) +  
+        (pow (T, 3)/450000.0);
+    /* Add the terms (pp. 144-6) */
     for (i=0; i<63; i++) {
-        xsum += X0*SINCOEFF[i][0];
-        xsum += X1*SINCOEFF[i][1];
-        xsum += X2*SINCOEFF[i][2];
-        xsum += X3*SINCOEFF[i][3];
-        xsum += X4*SINCOEFF[i][4];
-        psi     += (PE[i][0] + JC*PE[i][1])*sin(DEG_TO_RAD(xsum));
-        epsilon += (PE[i][2] + JC*PE[i][3])*cos(DEG_TO_RAD(xsum));
-        xsum=0;
+        PNsum += D  * COEFF[i][0];
+        PNsum += M  * COEFF[i][1];
+        PNsum += M1 * COEFF[i][2];
+        PNsum += F  * COEFF[i][3];
+        PNsum += O  * COEFF[i][4];
+        psi     += (PN[i][0] + JC*PN[i][1])*sin(DEG_TO_RAD(PNsum));
+        epsilon += (PN[i][2] + JC*PN[i][3])*cos(DEG_TO_RAD(PNsum));
+        PNsum=0;
     }
 
     deltaPsi = psi/36000000.0;
+    /* Nutation in obliquity */
     deltaEps = epsilon/36000000.0;
 
+
+    /* The obliquity of the ecliptic (p. 147, 22.3) */
     U = JM/10.0;
     E0 = 84381.448 - 4680.93 * U - 1.55 * pow(U,2) + 1999.25 * pow(U,3) 
         - 51.38 * pow(U,4)  - 249.67 * pow(U,5) - 39.05 * pow(U,6) + 7.12 
         * pow(U,7) + 27.87 * pow(U,8) + 5.79 * pow(U,9) + 2.45 * pow(U,10);
+    /* Real/true obliquity (p. 147) */
     E = E0/3600.0 + deltaEps;
-    lamda = G + deltaPsi + (-20.4898/(3600.0 * R));
+    rE = DEG_TO_RAD(E);
 
+    lambda = G + deltaPsi + (-20.4898/(3600.0 * R));
+    rLambda = DEG_TO_RAD(lambda);
+
+    /* Mean Sidereal time (p. 88) */
     V0 = 280.46061837 + 360.98564736629 * ( JD - 2451545) +  
         0.000387933 * pow(JC,2) - pow(JC,3)/ 38710000.0;
-    V = limitAngle(V0) + deltaPsi * cos(DEG_TO_RAD(E));
+    /* Apparent sidereal time */
+    V = limitAngle(V0) + deltaPsi * cos(rE);
 
-    RAn = sin(DEG_TO_RAD(lamda)) * cos(DEG_TO_RAD(E)) - 
-        tan(DEG_TO_RAD(Gg)) * sin(DEG_TO_RAD(E));
-    RAd = cos(DEG_TO_RAD(lamda));
+    RAn = sin(rLambda) * cos(rE) - tan(rGg) * sin(rE);
+    RAd = cos(rLambda);
     RA = limitAngle(RAD_TO_DEG(atan2(RAn,RAd)));
 
-    DEC = asin( sin(DEG_TO_RAD(Gg)) * cos(DEG_TO_RAD(E)) + 
-                            cos(DEG_TO_RAD(Gg)) * sin(DEG_TO_RAD(E)) * 
-                            sin(DEG_TO_RAD(lamda)));
+    DEC = asin( sin(rGg) * cos(rE) + cos(rGg) * sin(rE) * 
+                            sin(rLambda));
 
     astroday->ra = RA;
     astroday->dec = DEC;
@@ -655,33 +881,36 @@ void computeAstroDay(double JD, AstroDay* astroday)
 void computeTopAstro(const Location* loc, const Astro* astro, Astro* topAstro)
 {
     int i;
-    double lHour, SP;
-    double tU, tCos, tSin, tRA0 ,tRA ,tDEC;
+    double lHour, SP, rlHour, rLat;
+    double tU, tpCos, tpSin, tRA0 ,tRA ,tDEC;
+
+    rLat = DEG_TO_RAD(loc->degreeLat);
 
     for (i=0; i<3; i++)
     {
         lHour = limitAngle(astro->sid[i] + loc->degreeLong - astro->ra[i]);
+        rlHour = DEG_TO_RAD(lHour);
 
-        SP = 8.794/(3600 * astro->rsum[i]);
+        SP = DEG_TO_RAD (8.794/(3600 * astro->rsum[i]));
 
-        tU = atan (0.99664719 * tan(DEG_TO_RAD(loc->degreeLat)));
+	/* (p. 82, with b/a = 0.99664719) */
+	tU = atan (0.99664719 * tan(rLat));
 
-        tCos = cos(tU) + ( (loc->seaLevel)/6378140.0) * 
-            cos(DEG_TO_RAD(loc->degreeLat));
+        tpSin = 0.99664719 * sin(tU) + (loc->seaLevel/EARTH_RADIUS) * 
+            sin(rLat);
 
-        tSin = 0.99664719 * sin(tU) + ( loc->seaLevel/6378140.0) * 
-            sin(DEG_TO_RAD(loc->degreeLat));
+        tpCos = cos(tU) + (loc->seaLevel/EARTH_RADIUS) * cos(rLat);
 
-        tRA0 = (((-tCos) * sin(DEG_TO_RAD(SP)) * sin(DEG_TO_RAD(lHour))) 
-                / (cos(astro->dec[i]) - tCos * sin(DEG_TO_RAD(SP)) * cos(DEG_TO_RAD(lHour))));
+	/* (p. 297, 40.2) */
+        tRA0 = (((-tpCos) * sin(SP) * sin(rlHour)) / (cos(astro->dec[i]) - 
+                                                      tpCos * sin(SP) * cos(rlHour)));
 
+        tRA = astro->ra[i] + RAD_TO_DEG(tRA0);
 
-        tRA = astro->ra[i] +  RAD_TO_DEG(tRA0);
-
-
-        tDEC = RAD_TO_DEG(atan2((sin(astro->dec[i]) - tSin * sin(DEG_TO_RAD(SP))) * cos(tRA0), 
-                                cos(astro->dec[i]) - tCos * sin(DEG_TO_RAD(SP)) * 
-                                cos(DEG_TO_RAD(lHour))));
+	/* (p. 297, 40.3) */
+        tDEC = RAD_TO_DEG(atan2((sin(astro->dec[i]) - tpSin * sin(SP)) * cos(tRA0), 
+                                cos(astro->dec[i]) - tpCos * sin(SP) * 
+                                cos(rlHour)));
 
         topAstro->ra[i] = tRA;
         topAstro->dec[i] = tDEC;
@@ -693,7 +922,7 @@ void computeTopAstro(const Location* loc, const Astro* astro, Astro* topAstro)
 
 }
 
-double limitAngle(double L)
+static double limitAngle(double L)
 {
     double F;
     L /= 360.0;
@@ -706,7 +935,7 @@ double limitAngle(double L)
 }
 
 
-double limitAngle180(double L)
+static double limitAngle180(double L)
 {
     double F;
     L /= 180.0;
@@ -718,7 +947,8 @@ double limitAngle180(double L)
     else return L;
 }
 
-double limitAngle111(double L)
+/* Limit between 0 and 1 (fraction of day)*/
+static double limitAngle1(double L)
 {
     double F;
     F = L - floor(L);
@@ -727,7 +957,7 @@ double limitAngle111(double L)
     return F;
 }
 
-double limitAngle180between(double L)
+static double limitAngle180between(double L)
 {
     double F;
     L /= 360.0;
@@ -738,4 +968,47 @@ double limitAngle180between(double L)
         F -= 360;
     return F;
 }
-    
+
+/* FIXIT: Not used and not tested. */
+static double getYearFromJulian(double jd)
+{
+    double tempJD;
+    double F, Z, A, B, a, D, E;
+    int C, month, year;
+
+    tempJD = jd + 0.5;
+
+    /* Add an extra 0.5 to compensate for day starts if different than 0h UT at
+     * local time. */
+    if ((jd - floor(jd)) > 0)
+    {
+	tempJD -= (jd - floor(jd));
+	tempJD += 0.5;
+    }
+
+    Z = (int)tempJD;
+    F = tempJD - floor(tempJD);
+    if (Z < 2299161)
+	A = Z;
+    else
+    {
+	a = (int)((Z-1867216.25)/36524.25);
+	A = Z + 1 + a - (int)(a/4.0);
+    }
+
+    B = A + 1524;
+    C = (int)((B - 122.1)/365.25);
+    D = (int)(365.25 * C);
+    E = (int)((B - D)/30.6001);
+
+    if (E < 14)
+	month = E - 1;
+    else if ( E == 14 || E == 15 )
+	month = E - 13;
+    else month = 0; /* Bad month */
+
+    if (month > 2)
+	return C - 4716;
+    else return C - 4715;
+}
+
