@@ -1,10 +1,10 @@
 /************************************************************************
- * $Id: prayer.c,v 1.24 2005/03/14 22:09:07 thamer Exp $
+ * $Id: prayer.c 2186 2009-03-09 14:19:57Z thamer $
  *
  * ------------
  * Description:
  * ------------
- *  Copyright (c) 2003-2005, Arabeyes, Thamer Mahmoud
+ *  Copyright (c) 2003-2006, 2009, Arabeyes, Thamer Mahmoud
  *
  *  A full featured Muslim Prayer Times calculator
  *
@@ -12,10 +12,10 @@
  * -----------------
  * Revision Details:    (Updated by Revision Control System)
  * -----------------
- *  $Date: 2005/03/14 22:09:07 $
+ *  $Date: 2009-03-09 17:19:57 +0300 (Mon, 09 Mar 2009) $
  *  $Author: thamer $
- *  $Revision: 1.24 $
- *  $Source: /home/arabeyes/cvs/projects/itl/libs/prayertime/src/prayer.c,v $
+ *  $Revision: 2186 $
+ *  $Source$
  *
  * (www.arabeyes.org - under LGPL license - see COPYING file)
  ************************************************************************/
@@ -30,7 +30,7 @@
 #define DEF_NEAREST_LATITUDE 48.5
 #define DEF_IMSAAK_ANGLE 1.5
 #define DEF_IMSAAK_INTERVAL 10
-#define DEFAULT_ROUND_SEC 30 
+#define DEF_ROUND_SEC 30 
 #define AGGRESSIVE_ROUND_SEC 1
 
 
@@ -49,7 +49,7 @@ enum exmethods  { NONE_EX,
                   HALF_INVALID,
                   MIN_ALWAYS,
                   MIN_INVALID,
-                  GOOD_DIF };
+                  GOOD_INVALID_SAME };
 
 enum methods    { NONE,
                   EGYPT_SURVEY,
@@ -58,23 +58,19 @@ enum methods    { NONE,
                   NORTH_AMERICA,
                   MUSLIM_LEAGUE,
                   UMM_ALQURRA,
-                  FIXED_ISHAA };
+                  FIXED_ISHAA,
+                  EGYPT_NEW };
 
 enum salatType  { FAJR,
                   SHUROOQ,
-                  THUHR,
+                  ZUHR,
                   ASSR,
                   MAGHRIB,
                   ISHAA,
                   IMSAAK,
                   NEXTFAJR };
 
-
-Astro astroCache; /* This is Used for storing some formulae results between
-                   * multiple getPrayerTimes calls*/
-
-static double getThuhr (double lon, const Astro* astro);
-static double getShoMag ( const Location* loc, const Astro* astro, int type);
+static double getZuhr (double lon, const Astro* astro);
 static double getFajIsh (double Lat, double dec, double Ang);
 static double getAssr (double Lat, double dec, int mathhab);
 static void base6hm(double bs, const Location* loc, const Method* conf,
@@ -83,24 +79,27 @@ static void getDayInfo( const Date* date, double gmt, int *lastDay, double *juli
 static void getPrayerTimesByDay ( const Location* loc, const Method* conf, int lastDay,
                                   double julianDay, Prayer* pt, int type);
 
+Astro astroCache; /* This global variable is used for caching values between
+                   * multiple getPrayerTimesByDay() calls. You can disable this
+                   * caching feature by moving this line to the start of the
+                   * getPrayerTimesByDay() function. */
 
 void getPrayerTimes ( const Location* loc, const Method* conf, const Date* date, 
                       Prayer* pt)
 {  
     int lastDay;
     double julianDay;
-
     getDayInfo ( date, loc->gmtDiff, &lastDay, &julianDay);
     getPrayerTimesByDay( loc, conf, lastDay, julianDay, pt, 0);
 }
 
- 
 static void getPrayerTimesByDay ( const Location* loc, const Method* conf, 
                                   int lastDay, double julianDay, Prayer* pt, 
                                   int type)
 {
+
     int i, invalid;
-    double th, sh, mg, fj, is, ar;
+    double zu, sh, mg, fj, is, ar;
     double lat, lon, dec;
     double tempPrayer[6];
     Astro tAstro;
@@ -118,10 +117,10 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
     /* Get Prayer Times formulae results for this day of year and this
      * location. The results are NOT the actual prayer times */
     fj   = getFajIsh (lat, dec, conf->fajrAng);
-    sh   = getShoMag (loc, &tAstro, SHUROOQ);
-    th   = getThuhr (lon, &tAstro);
+    sh   = getSunrise(loc, &tAstro);
+    zu   = getZuhr (lon, &tAstro);
     ar   = getAssr (lat, dec, conf->mathhab);
-    mg   = getShoMag (loc, &tAstro, MAGHRIB);
+    mg   = getSunset(loc, &tAstro);
     is   = getFajIsh (lat, dec, conf->ishaaAng);
     
     /* Calculate all prayer times as Base-10 numbers in Normal circumstances */ 
@@ -130,25 +129,33 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
         tempPrayer[0] = 99;
         invalid = 1;
     } 
-    else tempPrayer[0] = th - fj;
+    else tempPrayer[0] = zu - fj;
      
     if (sh == 99)
         invalid = 1;
-
     tempPrayer[1] = sh;
-    tempPrayer[2] = th;
-    tempPrayer[3] = th + ar;
-    tempPrayer[4] = mg;
+
+    tempPrayer[2] = zu;
+
+    /* Assr */
+    if (ar == 99) {
+        tempPrayer[3] = 99;
+        invalid = 1;
+    } 
+    else tempPrayer[3] = zu + ar;
+
 
     if (mg == 99)
         invalid = 1;
+    tempPrayer[4] = mg;
+
 
     /* Ishaa */
     if (is == 99) {
         tempPrayer[5] = 99;
         invalid = 1;
     } 
-    else tempPrayer[5] = th + is;
+    else tempPrayer[5] = zu + is;
     
     
     /* Calculate all prayer times as Base-10 numbers in Extreme Latitudes (if
@@ -158,7 +165,6 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
     for (i=0; i<6; i++)
         pt[i].isExtreme = 0; 
      
-     
     if ((conf->extreme != NONE_EX) && !((conf->extreme == GOOD_INVALID || 
                                          conf->extreme == LAT_INVALID ||
                                          conf->extreme == SEVEN_NIGHT_INVALID ||
@@ -166,8 +172,8 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
                                          conf->extreme == HALF_INVALID) &&
                                         (invalid == 0)))
     {
-        double exdecPrev, exdecNext;
-        double exTh=99, exFj=99, exIs=99, exAr=99, exIm=99, exSh=99, exMg=99;
+        double exdecPrev, exdecNext, degnLat;
+        double exZu=99, exFj=99, exIs=99, exAr=99, exIm=99, exSh=99, exMg=99;
         double portion = 0;
         double nGoodDay = 0;
         int exinterval = 0;
@@ -182,25 +188,25 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
         case LAT_ALWAYS:
         case LAT_INVALID:
 
-            /* xxxthamer: we cannot compute this when interval is set because
+            /* FIXIT: we cannot compute this when interval is set because
              * angle==0 . Only the if-invalid methods would work */
             exLoc.degreeLat = conf->nearestLat;
             exFj = getFajIsh(conf->nearestLat, dec, conf->fajrAng);
             exIm = getFajIsh(conf->nearestLat, dec, conf->imsaakAng);
-            exIs = getFajIsh(conf->nearestLat, dec, conf->ishaaAng);
+            exSh = getSunrise(&exLoc, &tAstro);
             exAr = getAssr(conf->nearestLat, dec, conf->mathhab);
-            exSh = getShoMag (&exLoc, &tAstro, SHUROOQ);
-            exMg = getShoMag (&exLoc, &tAstro, MAGHRIB);
+            exMg = getSunset(&exLoc, &tAstro);
+            exIs = getFajIsh(conf->nearestLat, dec, conf->ishaaAng);
 
 
             switch(conf->extreme)
             {
             case LAT_ALL:
-                tempPrayer[0] = th - exFj;
+                tempPrayer[0] = zu - exFj;
                 tempPrayer[1] = exSh;
-                tempPrayer[3] = th + exAr;
+                tempPrayer[3] = zu + exAr;
                 tempPrayer[4] = exMg;
-                tempPrayer[5] = th + exIs;
+                tempPrayer[5] = zu + exIs;
                 pt[0].isExtreme = 1;
                 pt[1].isExtreme = 1;
                 pt[2].isExtreme = 1;
@@ -210,19 +216,19 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
                 break;
         
             case LAT_ALWAYS:
-                tempPrayer[0] = th - exFj;
-                tempPrayer[5] = th + exIs;
+                tempPrayer[0] = zu - exFj;
+                tempPrayer[5] = zu + exIs;
                 pt[0].isExtreme = 1;
                 pt[5].isExtreme = 1;
                 break;
         
             case LAT_INVALID:
                 if (tempPrayer[0] == 99) {
-                    tempPrayer[0] = th - exFj;
+                    tempPrayer[0] = zu - exFj;
                     pt[0].isExtreme = 1;
                 }
                 if (tempPrayer[5] == 99) {
-                    tempPrayer[5] = th + exIs;
+                    tempPrayer[5] = zu + exIs;
                     pt[5].isExtreme = 1;
                 }
                 break;
@@ -233,7 +239,7 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
         /* Nearest Good Day */
         case GOOD_ALL:
         case GOOD_INVALID:
-        case GOOD_DIF:
+        case GOOD_INVALID_SAME:
             
             exAstroPrev = astroCache;
             exAstroNext = astroCache;
@@ -242,22 +248,20 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
             for(i=0; i <= lastDay; i++)
             {
 
-                /* last closest day */
+                /* Last closest day */
                 nGoodDay = julianDay - i;
                 getAstroValuesByDay(nGoodDay, loc, &exAstroPrev, &tAstro);
                 exdecPrev = DEG_TO_RAD(tAstro.dec[1]);
                 exFj = getFajIsh(lat, exdecPrev, conf->fajrAng);
-
-                
                 if (exFj != 99)
                 {
                     exIs = getFajIsh(lat, exdecPrev, conf->ishaaAng);
                     if (exIs != 99)
                     {
-                        exTh = getThuhr (lon, &tAstro);
-                        exSh = getShoMag (loc, &tAstro, SHUROOQ);
-                        exMg = getShoMag (loc, &tAstro, MAGHRIB);
+                        exZu = getZuhr (lon, &tAstro);
+                        exSh = getSunrise (loc, &tAstro);
                         exAr = getAssr (lat, exdecPrev, conf->mathhab);
+                        exMg = getSunset (loc, &tAstro);
                         break;
                     }
                 }
@@ -272,10 +276,10 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
                     exIs = getFajIsh(lat, exdecNext, conf->ishaaAng);
                     if (exIs != 99)
                     {
-                        exTh = getThuhr (lon, &tAstro);
-                        exSh = getShoMag (loc, &tAstro, SHUROOQ);
-                        exMg = getShoMag (loc, &tAstro, MAGHRIB);
+                        exZu = getZuhr (lon, &tAstro);
+                        exSh = getSunrise (loc, &tAstro);
                         exAr = getAssr (lat, exdecNext, conf->mathhab);
+                        exMg = getSunset (loc, &tAstro);
                         break;
                     }
                 }
@@ -284,29 +288,33 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
             switch(conf->extreme)
             {
             case GOOD_ALL:
-                tempPrayer[0] = exTh - exFj;
+                tempPrayer[0] = exZu - exFj;
                 tempPrayer[1] = exSh;
-                tempPrayer[2] = exTh;
-                tempPrayer[3] = exTh + exAr;
+                tempPrayer[2] = exZu;
+                tempPrayer[3] = exZu + exAr;
                 tempPrayer[4] = exMg;
-                tempPrayer[5] = exTh + exIs;
+                tempPrayer[5] = exZu + exIs;
                 for (i=0; i<6; i++)
                     pt[i].isExtreme = 1;
                 break;
             case GOOD_INVALID:
                 if (tempPrayer[0] == 99) {
-                    tempPrayer[0] = exTh - exFj;
+                    tempPrayer[0] = exZu - exFj;
                     pt[0].isExtreme = 1;
                 }
                 if (tempPrayer[5] == 99) {
-                    tempPrayer[5] = exTh + exIs;
+                    tempPrayer[5] = exZu + exIs;
                     pt[5].isExtreme = 1;
                 }
                 break;
-
-            case GOOD_DIF:
-                /* Nearest Good Day: Different good days for Fajr and Ishaa (Not
-                 * implemented) */
+            case GOOD_INVALID_SAME:
+                if ((tempPrayer[0] == 99) || (tempPrayer[5] == 99))
+                    {
+                        tempPrayer[0] = exZu - exFj;
+                        pt[0].isExtreme = 1;
+                        tempPrayer[5] = exZu + exIs;
+                        pt[5].isExtreme = 1;
+                    }
                 break;
             }
             break;
@@ -318,7 +326,7 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
         case HALF_ALWAYS:
         case HALF_INVALID:
 
-            /* xxxthamer: For clarity, we may need to move the HALF_* methods
+            /* FIXIT: For clarity, we may need to move the HALF_* methods
              * into their own separate case statement. */    
             switch(conf->extreme)
             {
@@ -392,40 +400,44 @@ static void getPrayerTimesByDay ( const Location* loc, const Method* conf,
             break;
         } /* end switch */
     } /* end extreme */
-
     
     /* Apply intervals if set */
     if (conf->extreme != MIN_INVALID && 
         conf->extreme != HALF_INVALID &&
-        conf->extreme != HALF_ALWAYS) {
-        if (conf->fajrInv != 0)
-            tempPrayer[0] = tempPrayer[1] - (conf->fajrInv / 60.0);    
-        if (conf->ishaaInv != 0)
-            tempPrayer[5] = tempPrayer[4] + (conf->ishaaInv / 60.0); 
-    }
+        conf->extreme != HALF_ALWAYS) 
+    {
+        if (conf->fajrInv != 0) {
+            if (tempPrayer[1] != 99)
+                tempPrayer[0] = tempPrayer[1] - (conf->fajrInv / 60.0);
+            else tempPrayer[0] = 99;
+        }
 
+        if (conf->ishaaInv != 0) {
+            if (tempPrayer[4] != 99)
+                tempPrayer[5] = tempPrayer[4] + (conf->ishaaInv / 60.0);
+            else tempPrayer[5] = 99;
+        }
+    }
  
     /* Final Step: Fill the Prayer array by doing decimal degree to
-     * Prayer structure conversion*/
+     * Prayer structure conversion */
     if (type == IMSAAK || type == NEXTFAJR)
         base6hm(tempPrayer[0], loc, conf, &pt[0], type);
     else {
         for (i=0; i<6; i++)
             base6hm(tempPrayer[i], loc, conf, &pt[i], i);
     }
-
 }
 
-
-
 static void base6hm(double bs, const Location* loc, const Method* conf, 
                     Prayer* pt, int type)
 {
     double min, sec;
 
+    /* Set to 99 and return if prayer is invalid */
     if (bs == 99)
     {
-        pt->hour    = 99;
+        pt->hour = 99;
         pt->minute = 99;
         pt->second = 0;
         return;
@@ -438,7 +450,6 @@ static void base6hm(double bs, const Location* loc, const Method* conf,
         else  bs += (conf->offList[type] / 60.0);
     }
 
-
     /* Fix after minus offsets before midnight */
     if (bs < 0) {
         while (bs < 0)
@@ -448,11 +459,10 @@ static void base6hm(double bs, const Location* loc, const Method* conf,
     min = (bs - floor(bs)) * 60;
     sec = (min - floor(min)) * 60;
 
-
     /* Add rounding minutes */
     if (conf->round == 1)
     {
-        if (sec >= DEFAULT_ROUND_SEC)
+        if (sec >= DEF_ROUND_SEC)
             bs += 1/60.0;
         /* compute again */
         min = (bs - floor(bs)) * 60;
@@ -463,14 +473,14 @@ static void base6hm(double bs, const Location* loc, const Method* conf,
         switch(type)
         {
         case FAJR:
-        case THUHR:
+        case ZUHR:
         case ASSR:
         case MAGHRIB:
         case ISHAA:
         case NEXTFAJR:
             
             if (conf->round == 2) {
-                if (sec >= DEFAULT_ROUND_SEC) {
+                if (sec >= DEF_ROUND_SEC) {
                     bs += 1/60.0;
                     min = (bs - floor(bs)) * 60;
                 }
@@ -499,7 +509,6 @@ static void base6hm(double bs, const Location* loc, const Method* conf,
     pt->hour   = (int)bs;
     pt->minute = (int)min;
     pt->second = (int)sec;
-
 }
 
 void getImsaak (const Location* loc, const Method* conf, const Date* date, 
@@ -529,7 +538,7 @@ void getImsaak (const Location* loc, const Method* conf, const Date* date,
     getDayInfo ( date, loc->gmtDiff, &lastDay, &julianDay);
     getPrayerTimesByDay( loc, &tmpConf, lastDay, julianDay, temp, IMSAAK);
 
-    /* xxxthamer: We probably need to check whether it's possible to compute
+    /* FIXIT: We probably need to check whether it's possible to compute
      * Imsaak normally for some extreme methods first */
     /* In case of an extreme Fajr time calculation use intervals for Imsaak and
      * compute again */
@@ -563,149 +572,64 @@ void getNextDayImsaak (const Location* loc, const Method* conf, const Date* date
     getImsaak (loc, conf, &tempd, &temppt);
 
     *pt = temppt; 
-
 }
 
 void getNextDayFajr (const Location* loc, const Method* conf, const Date* date, 
                      Prayer* pt)
 {
-
     Prayer temp[6];
     int lastDay;
     double julianDay;
-
     getDayInfo ( date, loc->gmtDiff, &lastDay, &julianDay);
     getPrayerTimesByDay( loc, conf, lastDay, julianDay+1, temp, NEXTFAJR);
  
     *pt = temp[0]; 
 }
 
-
-static double getFajIsh(double Lat, double dec, double Ang)
+static double getFajIsh(double lat, double dec, double Ang)
 {
+    double rlat = DEG_TO_RAD(lat);
 
-    double part1 = cos(DEG_TO_RAD(Lat)) * cos(dec);
-    double part2 = -sin(DEG_TO_RAD(Ang)) - sin(DEG_TO_RAD(Lat))
-        * sin(dec);
+    /* Compute the hour angle */
+    double part1 = sin(DEG_TO_RAD(-Ang)) - (sin (rlat) * sin (dec));
+    double part2 = cos (rlat) * cos (dec);
+    double part3 = part1 / part2;
     
-    double part3 = part2 / part1;
-    if ( part3 <= INVALID_TRIGGER)
+    if ( part3 < -INVALID_TRIGGER || part3 > INVALID_TRIGGER)
         return 99;
 
     return DEG_TO_10_BASE * RAD_TO_DEG (acos(part3) );
-
 }
 
-
-static double getShoMag (const Location* loc, const Astro* astro, int type)
+static double getZuhr(double lon, const Astro* astro)
 {
-    double lhour, M, sidG, ra0=astro->ra[0], ra2=astro->ra[2];
-    double A, B, H, sunAlt, R, tH;
-
-    double part1 = sin (DEG_TO_RAD (loc->degreeLat)) * sin (DEG_TO_RAD(astro->dec[1]));
-    double part2a =  CENTER_OF_SUN_ANGLE;
-    double part2 = sin (DEG_TO_RAD (part2a)) - part1;
-    double part3 = cos (DEG_TO_RAD (loc->degreeLat)) * cos (DEG_TO_RAD(astro->dec[1]));
-
-    double part4 = part2 / part3;
-    
-    if (part4 <= -1 || part4 >= 1)
-        return 99;
-
-    lhour =  limitAngle180 (( RAD_TO_DEG (acos (part4))));
-    M = ((astro->ra[1] - loc->degreeLong - astro->sid[1]) / 360.0);
-
-    if (type ==  SHUROOQ)
-        M = M - (lhour/360.0);
-    if (type == MAGHRIB)
-        M = M + (lhour/360.0);
-        
-    M = limitAngle111(M);
-
-    sidG = limitAngle(astro->sid[1] + 360.985647 * M);
-
-    ra0 = astro->ra[0];
-    ra2 = astro->ra[2];
-
-    if (astro->ra[1] > 350 && astro->ra[2] < 10)
-        ra2 += 360;
-    if (astro->ra[0] > 350 && astro->ra[1] < 10)
-        ra0 = 0;
-
-    A = astro->ra[1] + (M * (( astro->ra[1] - ra0) +
-                             (ra2 - astro->ra[1] ) +
-                             (( ra2 - astro->ra[1] ) -
-                              ( astro->ra[1]  -  ra0)) * M) / 2.0 );
-
-    B = astro->dec[1] + (M * ((astro->dec[1] - astro->dec[0]) + 
-                              (astro->dec[2] - astro->dec[1]) + 
-                              ((astro->dec[2] - astro->dec[1]) -  
-                               (astro->dec[1] - astro->dec[0])) * M) / 2.0 );
-
-    H =  limitAngle180between(sidG +  loc->degreeLong -  A);
-
-    tH =  H - RAD_TO_DEG(astro->dra[1]);
-
-    sunAlt = RAD_TO_DEG(asin (  sin(DEG_TO_RAD( loc->degreeLat)) * sin(DEG_TO_RAD (B)) 
-                                + cos(DEG_TO_RAD( loc->degreeLat)) * cos(DEG_TO_RAD (B)) 
-                                * cos(DEG_TO_RAD(tH)) ));
-
-    sunAlt += getRefraction(loc, sunAlt);
-    
-    R = (M + (( sunAlt - CENTER_OF_SUN_ANGLE+ (ALTITUDE_REFRACTION * 
-                                               pow (loc->seaLevel,0.5)))
-              /(360.0 * cos(DEG_TO_RAD (B)) *  cos(DEG_TO_RAD( loc->degreeLat)) * 
-                                               sin(DEG_TO_RAD (tH)))));
-
-    return  (R * 24.0);
-
+    return getTransit(lon, astro);
 }
 
-static double getThuhr(double lon, const Astro* astro)
+static double getAssr(double lat, double dec, int mathhab)
 {
+    double part1, part2, part3, part4, ndec;
+    double rlat = DEG_TO_RAD(lat);
 
-    double M, sidG;
-    double ra0=astro->ra[0], ra2=astro->ra[2];
-    double A, H;
-
-    M = ((astro->ra[1] - lon - astro->sid[1]) / 360.0);
-    M = limitAngle111(M);
-    sidG =  astro->sid[1] + 360.985647 * M;
-
-    if (astro->ra[1] > 350 && astro->ra[2] < 10)
-        ra2 += 360;
-    if (astro->ra[0] > 350 && astro->ra[1] < 10)
-        ra0 = 0;
-
-    A = astro->ra[1] + (M * ((astro->ra[1] - ra0) 
-                             + ( ra2 - astro->ra[1]) + 
-                             (( ra2 - astro->ra[1]) -  
-                              (astro->ra[1] - ra0)) * M) / 2.0 );
-
-    H =  limitAngle180between(sidG + lon - A);
-
-    return  24.0 * (M - H/360.0);
-}
-
-static double getAssr(double Lat, double dec, int mathhab)
-{
-    double part1, part2, part3, part4;
+    /* Reverse if at or near the southern hemisphere */    
+    ndec = dec;
+    if (lat < 0.0)
+	ndec = -dec;
+    part1 = mathhab + tan(rlat - ndec);
+    if (part1 < 1.0)
+	part1 = mathhab - tan(rlat - ndec);
     
-    part1 = mathhab + tan(DEG_TO_RAD(Lat) - dec);
-    if (part1 < 1 || Lat < 0)
-        part1 = mathhab - tan(DEG_TO_RAD(Lat) - dec);
-
     part2 = (PI/2.0) - atan(part1);
-    part3 = sin(part2) - sin(DEG_TO_RAD(Lat)) * sin(dec);
-    part4 = (part3 / (cos(DEG_TO_RAD(Lat)) * cos(dec)));
+    /* Compute the hour angle */
+    part3 = sin(part2) - (sin(rlat) * sin(ndec));
+    part4 = (part3 / (cos(rlat) * cos(ndec)));
 
-/*  if (part4 > 1) */
-/*      return 99; */
+    if ( part4 < -INVALID_TRIGGER || part4 > INVALID_TRIGGER)
+	return 99;
 
     return DEG_TO_10_BASE * RAD_TO_DEG (acos(part4));
 }
 
- 
 int getDayofYear(int year, int month, int day)
 {
     int i;
@@ -719,7 +643,6 @@ int getDayofYear(int year, int month, int day)
 
     for (i=1; i<month; i++)
         day += dayList[isLeap][i];
-  
     
     return day;
 }
@@ -816,6 +739,11 @@ void getMethod(int n, Method* conf)
         conf->ishaaAng = 0.0;
         conf->ishaaInv = 90;
         break;
+
+    case EGYPT_NEW:
+        conf->fajrAng = 19.5;
+        conf->ishaaAng = 17.5;
+        break;
     }
 }
 
@@ -823,7 +751,7 @@ void getMethod(int n, Method* conf)
  * great circle formula */ 
 double getNorthQibla(const Location* loc)
 {
-    /* xxxthamer: reduce DEG_TO_RAD usage */
+    /* FIXIT: reduce DEG_TO_RAD usage */
     double num, denom;
     num = sin (DEG_TO_RAD (loc->degreeLong) - DEG_TO_RAD (KAABA_LONG));
     denom = (cos (DEG_TO_RAD (loc->degreeLat)) * tan (DEG_TO_RAD (KAABA_LAT))) -
